#!/usr/bin/perl 


#####################################
#Turn On/Off - Drain Up/Down Manager
#####################################



#####################################################################
#####################################################################

use warnings;
use strict;
use DBI;
use POSIX;

#2.0.5 - Add ipv6

####################### CONFIGURATION #######################
use XML::Simple;
use Data::Dumper;
my $conffile="/etc/lvs/lvs.conf.xml";

############
#Config Vars
############
my $conffile="/etc/mylvs/mylvs.conf.xml";
my ($servername,$database,$sqluser,$sqlpassword,$lvscmd,$bkupdatabase,$bkupsqluser,$bkupsqlpassword,$bkupservername,$ActiveWarning,$InActiveWarning,$HTTP_ERR,$HTTP_WARN,$simulate,$maxcount,$backuppath,$restoreapp,$backupapp,$SuppressedAddress,$HTMLLVSView,$bkupreplcommand,$sitelogo);
&getconfig;
############



####################################################################
my $basepath=$ENV{DOCUMENT_ROOT};
my $server = $ENV{HTTP_HOST};
####################################################################


my $retscriptname="mylvs.pl";
#Remember this:   <td colspan="2" align="left" valign="top"><a href="javascript:history.back()"><img src="/images/back-tab.gif" alt="previous page" width="63" height="29" border="0"></a></td>

####################### CONFIGURATION #######################
my $version = "2.0.5";

#Get Apache Environs
my $tparam=$ENV{QUERY_STRING};
my $curuser=$ENV{REMOTE_USER};
my $curip=$ENV{REMOTE_ADDR};
my $granted = undef;   #user login flag - default off
#For static test purposes ...
#$tparam = "to=2";
#$tparam = "wu=1";
#$tparam = "wd=1";
#$tparam = "to=1";
#$tparam = "td=4";
#####################################################################
#####################################################################

my ($sqlAlias,$sqlIP,$GlobalAlias,$GlobalIP,$GlobalTxt,$replstr,$weightdownstate,$myCurID);
my (%ResInvID,$ResWeight,$dbh,$mstring,$mxstring,$var,$value,$auto_ok);
my ($NodeService,$NodeVip,$sqlnoden,$sqlcur,$sqlInvID,$sqlInvTID,$sqlInvWeight,$sqlNodeID,$sqlLink,$sqlIPort,$sqlinv,$sqlalias,$sqlactive,$sqlmethod,$sqlweight,$sqlenabled,$sqlPoolID,$sqlWeightStat,$curstat,$sqlpool,$sqlPoolReource,$sqlAliasEnabled,$mdstring);
my $showid;  #=1;  #Site TestID !!!
my ($myIP,$myHost,$myCHost);
my ($hdeb,$NodeVip6,$mstring6,$sqlIP6,$GlobalIP6,$myIP6,$srvCnt4,$srvCnt6); #2.0.5
my $prtDeb=undef;  #2.0.5 - set to undef for non debug


$granted=&chkuser($curuser);  #Check User

$srvCnt4=0;
$srvCnt6=0;

print &HTMLHeader;
$dbh = DBI->connect("DBI:mysql:$database:$servername:3306",$sqluser,$sqlpassword); 

if ($tparam =~ m/^(\w{2})=(\d*)/){  
	$showid=$2;
  $sqlalias = $dbh->prepare("SELECT alias,realip,realip6 FROM tblResources where resID = '" . $showid . "';" ); 
  $sqlalias->execute ; #Get Inventory and ID's
  (($sqlAlias,$sqlIP,$sqlIP6)=$sqlalias->fetchrow_array);
  $GlobalAlias=$sqlAlias;
  $GlobalIP=$sqlIP;
  $GlobalIP6=$sqlIP6;
  $hdeb.="$GlobalIP - $GlobalIP6<br>";
}  

if ($tparam =~ m/^td=/){  
 $GlobalTxt="Turn off services";
}

if ($tparam =~ m/^tu=/){  
 $GlobalTxt="Turn on services and weight them up";
}

if ($tparam =~ m/^wu=/){  
 $GlobalTxt="Weight up services";
}

if ($tparam =~ m/^wd=/){  
 $GlobalTxt="Weight down services";
}

print &HTMLStartOutsideTable;
print &HTMLStartInsideTable;
 
if ($granted) {
 #########
 #TURN OFF
 #########
 if ($tparam =~ m/^td=(\d*)/){  
 	if ($simulate eq "1") {
 	 &PrintHTMLMessage("Simualtation Mode: !!<BR>")
 	}	

	$showid=$1;
  $sqlalias = $dbh->prepare("SELECT alias,realip,realip6 FROM tblResources where resID = '" . $showid . "';" ); 
  $sqlalias->execute ; #Get Inventory and ID's
  (($sqlAlias,$sqlIP,$sqlIP6)=$sqlalias->fetchrow_array);
  $myIP=$sqlIP;
  $myIP6=$sqlIP6;
  
  $hdeb.="$myIP - $myIP6<br>";
  $myHost=$sqlAlias;
  $sqlinv = $dbh->prepare("SELECT invID,weight,nodeID,link,port,active,method,weight,automovepoolid FROM tblNodesInventory where resID = '" . $showid . "';" ); 
  $sqlinv->execute ; #Get Inventory and ID's
  while (($sqlInvID,$sqlInvWeight,$sqlNodeID,$sqlLink,$sqlIPort,$sqlactive,$sqlmethod,$sqlweight,$sqlPoolID)=$sqlinv->fetchrow_array) {
  #$sqlinv = $dbh->prepare("SELECT invID,weight,nodeID,link,port,automovepoolid FROM tblNodesInventory where resID = '" . $showid . "';" ); 
  #$sqlinv->execute ; #Get Inventory and ID's
  #while (($sqlInvID,$sqlInvWeight,$sqlNodeID,$sqlLink,$sqlIPort,$sqlPoolID)=$sqlinv->fetchrow_array) {
   $sqlnoden = $dbh->prepare("SELECT service,vip,vip6 FROM tblNodes where nodeid = '" . $sqlNodeID . "';" ); 
   $sqlnoden->execute ; 
   $hdeb.="SELECT service,vip,vip6 FROM tblNodes where nodeid = '" . $sqlNodeID . "<br>";
   ($NodeService,$NodeVip,$NodeVip6)=$sqlnoden->fetchrow_array ;
   if ($NodeService !~ /fwm/i) {
    
    $hdeb.="$NodeVip<-->$sqlIP // $NodeVip6<-->$sqlIP6<br>";
    if (($NodeVip) and ($sqlIP)){
     $mstring = "$lvscmd -d -t $NodeVip -r $sqlIP:$sqlIPort";
     $srvCnt4++;
    } 
    if (($NodeVip6) and ($sqlIP6)){
     #Apply -0: /usr/bin/sudo /sbin/ipvsadm -d -t fde2:8acd:e9d3::220:7798 -r fde2:8acd:e9d3::221:7798
     if ($NodeVip6=~/(.*):(\d+)$/) {
      $mstring6 = "$lvscmd -d -t [$1]:$2 -r [$sqlIP6]:$sqlIPort"; #2.0.5
      $srvCnt6++;
     } 
    } 
   } else {
    if (($NodeVip) and ($sqlIP)){
     $mstring = "$lvscmd -d -f $NodeVip -r $sqlIP:$sqlIPort";	
     $srvCnt4++;
    } 
    if (($NodeVip6) and ($sqlIP6)){ #
     $mstring6 = "$lvscmd -d -f $NodeVip6 -6 -r [$sqlIP6]:$sqlIPort";	 #2.0.5
     $srvCnt6++;
    } 
   }	
   #	APPLY: /usr/bin/sudo /sbin/ipvsadm -a -t 10.0.1.4:7000 -r 10.0.1.11:7000 -g -w 10
   #	APPLY: /usr/bin/sudo /sbin/ipvsadm -a -f 5 -r 10.0.1.11 -g -w 100
   #	APPLY: /usr/bin/sudo /sbin/ipvsadm -d -t 10.0.1.4:7000 -r 10.0.1.11:7000
   #	APPLY: /usr/bin/sudo /sbin/ipvsadm -d -f 5 -r 10.0.1.11
   
   if ($mstring) {
    &PrintHTMLMessage ("Apply -$sqlPoolID: $mstring<br>");
    &lvslog(2,"Turn off service by: $mstring",$curuser);
   } 

   if ($mstring6) {
    &PrintHTMLMessage ("Apply -$sqlPoolID: $mstring6<br>");
    &lvslog(2,"Turn off service by: $mstring6",$curuser);
   } 
   
   if ($simulate eq "0") {
    if ($mstring) {
     `$mstring`;
    } 
    if ($mstring6) {
    	`$mstring6`;
    }	
   } 
  
   
   &PrintHTMLMessage("SQL:: UPDATE tblNodesInventory SET enabled = '0' where invID = $sqlInvID<br>");
   if ($simulate eq "0") {
    $sqlcur = $dbh->prepare("UPDATE tblNodesInventory SET enabled = '0' where invID ='" . $sqlInvID . "';" );
    $sqlcur->execute;
   } 
   $auto_ok=1; 
   ##########################################
   ##############   AUTOMOVE   ##############
   ##########################################
   
   if (($sqlPoolID) and ($sqlPoolID ne "0")) {
   	$auto_ok=undef;
    &PrintHTMLMessage("ATTENTION: Automove Service Applied To Pool: $sqlPoolID <BR>");
    &PrintHTMLMessage("SQL: SELECT resID FROM tblPoolResources where poolID = '" . $sqlPoolID . "' <BR>");
    
    $sqlpool = $dbh->prepare("SELECT resID FROM tblPoolResources where poolID = '" . $sqlPoolID . "';" ); 
    $sqlpool->execute ; #Get Inventory and ID's
    while (($sqlPoolReource)=$sqlpool->fetchrow_array) {
     if ($sqlPoolReource ne $showid) {
      $sqlalias = $dbh->prepare("SELECT realip,enabled,alias FROM tblResources where resID = '" . $sqlPoolReource . "';" ); 
      $sqlalias->execute ; #Get Inventory and ID's
      (my($sqlIP,$sqlAliasEnabled,$sqlHostName)=$sqlalias->fetchrow_array);
      $weightdownstate=undef;
      $sqlinv = $dbh->prepare("SELECT invID FROM tblNodesInventory where weightdown = '1' and resID = '" . $sqlPoolReource . "';" ); 
      $sqlinv->execute ; #Get Inventory and ID's
      while (($sqlInvTID)=$sqlinv->fetchrow_array) {
        $weightdownstate=1;  
      }
      if (($sqlAliasEnabled eq "1") and ( !$weightdownstate)) {
      $auto_ok=1;
       &PrintHTMLMessage("Resource is good for takeover $sqlIP - $sqlHostName<BR>");
       $myCHost=$sqlHostName;
    	 if ($NodeService !~ /fwm/i) {
        $mxstring = "$lvscmd -a -t $NodeVip -r $sqlIP:$sqlIPort -$sqlmethod -w $sqlweight";
       } else {  
        $mxstring = "$lvscmd -a -f $NodeVip -r $sqlIP -$sqlmethod -w $sqlweight";	
       }	
       &PrintHTMLMessage("ReApply: $mxstring <BR>");
       if ($simulate eq "0") {
        `$mxstring`;
       } 
       &lvslog(2,"Move service by: $mxstring",$curuser);
       #Set New Node
       &PrintHTMLMessage(" SQL: UPDATE tblNodesInventory SET resID = '" . $sqlPoolReource . "' where invID ='" . $sqlInvID . "' <BR>");       
       if ($simulate eq "0") {
        $sqlcur = $dbh->prepare("UPDATE tblNodesInventory SET resID = '" . $sqlPoolReource . "' where invID ='" . $sqlInvID . "';" );
        $sqlcur->execute;
       } 
       &PrintHTMLMessage(" SQL: UPDATE tblNodesInventory SET enabled = '1' where invID ='" . $sqlInvID . "' <BR>");              
       if ($simulate eq "0") {
        $sqlcur = $dbh->prepare("UPDATE tblNodesInventory SET enabled = '1' where invID ='" . $sqlInvID . "';" );
        $sqlcur->execute;
       } 
       ($var,$value) = split(/$myHost\s*/,$sqlLink,2);     #Get Full Command 
       &PrintHTMLMessage(" SQL: UPDATE tblNodesInventory SET link = '".$var.$myCHost.$value."' where invID ='" . $sqlInvID . "' <BR>");
       if ($simulate eq "0") {
        $sqlcur = $dbh->prepare("UPDATE tblNodesInventory SET link = '".$var.$myCHost.$value."' where invID ='" . $sqlInvID . "';" );
        $sqlcur->execute;
       } 
       last;
      } else {
      	&PrintHTMLMessage("Warning: Resource is bad for takeover $sqlIP - $sqlHostName<BR>");
      }
     }	#if ($sqlPoolReource ne $showid)  
    }   #while (($sqlPoolReource)=$sqlpool->fetchrow_array) 	    
   }    #if (($sqlPoolID) and ($sqlPoolID ne "0"))
   ##########################################
   ###########  DONE AUTOMOVE  ##############
   ##########################################
  }   #while (my($sqlInvID,$sqlInvWeight,$sqlNodeID,$sqlLink,$sqlIPort)=$sqlinv->fetchrow_array) 
  if (!$auto_ok) {
   &PrintHTMLMessage("==>> CRITICAL <<== - No Server found for automove <BR>");
   #SendEmergencyEMail
  }
  &PrintHTMLMessage("SQL: UPDATE tblResources SET enabled = '0' where resID = '" . $showid . "' <BR>");
  if ($simulate eq "0") {
   $sqlcur = $dbh->prepare("UPDATE tblResources SET enabled = '0' where resID = '" . $showid . "';");
   $sqlcur->execute;
  } 
  &PrintHTMLMessage(" SQL: UPDATE tblState SET statechange = '1' where stateID ='1' <BR>");
  if ($simulate eq "0") {
   $sqlcur = $dbh->prepare("UPDATE tblState SET statechange = '1' where stateID ='1';");
   $sqlcur->execute;
  } 
  
  print "Services IPv4: $srvCnt4<br>\n";
  print "Services IPv6: $srvCnt6<br>\n";
  print "Turn-Off Proces Complete!\n";
 }    #if ($tparam =~ m/^turnoff=(\d*)/){  

 ########
 #TURN ON
 ########

 if ($tparam =~ m/^tu=(\d*)/){  
 	if ($simulate eq "1") {
 	 &PrintHTMLMessage("Simualtation Mode: !!<BR>");
 	}	

	$showid=$1;
  $sqlalias = $dbh->prepare("SELECT alias,realip,realip6 FROM tblResources where resID = '" . $showid . "';" ); 
  $sqlalias->execute ; #Get Inventory and ID's
  (($sqlAlias,$sqlIP,$sqlIP6)=$sqlalias->fetchrow_array);
  $myIP=$sqlIP;
  $myIP6=$sqlIP6;
  $sqlinv = $dbh->prepare("SELECT invID,weight,nodeID,link,port,active,method,weight FROM tblNodesInventory where resID = '" . $showid . "';" ); 
  $sqlinv->execute ; #Get Inventory and ID's
  while (($sqlInvID,$sqlInvWeight,$sqlNodeID,$sqlLink,$sqlIPort,$sqlactive,$sqlmethod,$sqlweight)=$sqlinv->fetchrow_array) {
   $sqlnoden = $dbh->prepare("SELECT service,vip,vip6 FROM tblNodes where nodeid = '" . $sqlNodeID . "';" ); 
   $sqlnoden->execute ; 
   ($NodeService,$NodeVip,$NodeVip6)=$sqlnoden->fetchrow_array ;
   if ($NodeService !~ /fwm/i) {
    if ($NodeVip) {
     $mstring = "$lvscmd -a -t $NodeVip -r $sqlIP:$sqlIPort -$sqlmethod -w $sqlweight";
     $srvCnt4++;
    } 

    if ($NodeVip6) {
     if ($NodeVip6=~/(.*):(\d+)$/) {  #
      $mstring6 = "$lvscmd -a -t [$1]:$2 -r [$sqlIP6]:$sqlIPort -$sqlmethod -w $sqlweight";
      $srvCnt6++;
     } 
    } 
   } else {
    if ($NodeVip) {
     $mstring = "$lvscmd -a -f $NodeVip -r $sqlIP:$sqlIPort -$sqlmethod -w $sqlweight";	
     $srvCnt4++;
    } 
    
    if ($NodeVip6) { 
     #$mstring6 = "$lvscmd -d -f $NodeVip6 -6 -r [$sqlIP6]:$sqlIPort";	 #2.0.5
     $mstring6 = "$lvscmd -a -f $NodeVip6 -6 -r [$sqlIP6]:$sqlIPort -$sqlmethod -w $sqlweight";	
     $srvCnt6++;
    }  
   }	
   if ($sqlactive eq "0") {
    &PrintHTMLMessage("WARNING: Host $sqlIP:$sqlIPort is in down state. <BR>");
   }

   if ($mstring) {
    &PrintHTMLMessage ("Apply: $mstring <BR>");
    &lvslog(2,"Turn on service by: $mstring",$curuser);
   } 
   if ($mstring6) {
   	&PrintHTMLMessage ("Apply: $mstring6 <BR>");
   	&lvslog(2,"Turn on service by: $mstring6",$curuser);
   } 


   if ($simulate eq "0") {
    if ($mstring) {
     `$mstring`;
    } 
    if ($mstring6) {
     `$mstring6`;
    } 
   } 

   if ($simulate eq "0") {
    $sqlcur = $dbh->prepare("UPDATE tblNodesInventory SET enabled = '1', active='1' where invID ='" . $sqlInvID . "';" );
    $sqlcur->execute;
   } 
   &PrintHTMLMessage(" SQL: UPDATE tblMonitor SET status='UNKNOWN' where invID ='" . $sqlInvID . "' and enabled ='1' <BR>");
   if ($simulate eq "0") {
    $sqlcur = $dbh->prepare("UPDATE tblMonitor SET status='UNKNOWN' where invID ='" . $sqlInvID . "' and enabled ='1';" );
    $sqlcur->execute;
   } 
   &PrintHTMLMessage(" SQL: UPDATE tblNodesInventory SET weightdown = '0' where invID ='" . $sqlInvID . "' <BR>");
   if ($simulate eq "0") {
    $sqlcur = $dbh->prepare("UPDATE tblNodesInventory SET weightdown = '0' where invID ='" . $sqlInvID . "';" );
    $sqlcur->execute;
   } 
   
  }   #while (my($sqlInvID,$sqlInvWeight,$sqlNodeID,$sqlLink,$sqlIPort)=$sqlinv->fetchrow_array) 
   &PrintHTMLMessage("SQL: UPDATE tblResources SET enabled = '1' where resID = '" . $showid . "' <BR>");
   if ($simulate eq "0") {
    $sqlcur = $dbh->prepare("UPDATE tblResources SET enabled = '1' where resID = '" . $showid . "';");
    $sqlcur->execute;
   } 
   &PrintHTMLMessage("SQL: UPDATE tblState SET statechange = '1' where stateID ='1' <BR>");
   if ($simulate eq "0") {
    $sqlcur = $dbh->prepare("UPDATE tblState SET statechange = '1' where stateID ='1';");
    $sqlcur->execute;
   } 
  print "Services IPv4: $srvCnt4<br>\n";
  print "Services IPv6: $srvCnt6<br>\n";

   print "Turn-On Proces Complete!\n";
 }    #if ($tparam =~ m/^turnoff=(\d*)/){  





 #############
 #WEIGHT UP
 #############
 if ($tparam =~ m/^wu=(\d*)/){  
 	if ($simulate eq "1") {
 		&PrintHTMLMessage("Simualtation Mode: !!<BR>");
 	}	

	$showid=$1;
  $sqlalias = $dbh->prepare("SELECT alias,realip,realip6 FROM tblResources where resID = '" . $showid . "';" ); 
  $sqlalias->execute ; #Get Inventory and ID's
  (($sqlAlias,$sqlIP,$sqlIP6)=$sqlalias->fetchrow_array);
  $myIP=$sqlIP;
  $myIP6=$sqlIP6;
  $sqlinv = $dbh->prepare("SELECT invID,weight,nodeID,link,port,active,method,weight,enabled FROM tblNodesInventory where resID = '" . $showid . "';" ); 
  $sqlinv->execute ; #Get Inventory and ID's
  while (($sqlInvID,$sqlInvWeight,$sqlNodeID,$sqlLink,$sqlIPort,$sqlactive,$sqlmethod,$sqlweight,$sqlenabled)=$sqlinv->fetchrow_array) {
   $sqlnoden = $dbh->prepare("SELECT service,vip,vip6 FROM tblNodes where nodeid = '" . $sqlNodeID . "';" ); 
   $sqlnoden->execute ; 
   ($NodeService,$NodeVip,$NodeVip6)=$sqlnoden->fetchrow_array ;
   if ($NodeService !~ /fwm/i) {
    
    if (($NodeVip) and ($sqlIP)){
     $mstring = "$lvscmd -e -t $NodeVip -r $sqlIP:$sqlIPort -$sqlmethod -w $sqlweight";
     $srvCnt4++;
    }
    $hdeb.="$NodeVip6-$sqlIP6<br>";
    if (($NodeVip6) and ($sqlIP6)){
     if ($NodeVip6=~/(.*):(\d+)$/) {  #
      $mstring6 = "$lvscmd -e -t [$1]:$2 -r [$sqlIP6]:$sqlIPort -$sqlmethod -w $sqlweight";
      $srvCnt6++;
     } 
    }
    
    
     
   } else {
   	if ($NodeVip) {
     $mstring = "$lvscmd -e -f $NodeVip -r $sqlIP -$sqlmethod -w $sqlweight";	
     &lvslog(2,"Weight up service by: $mstring",$curuser);
    } 
  
   	if ($NodeVip6) {
     $mstring6 = "$lvscmd -e -f $NodeVip6 -r [$sqlIP] -$sqlmethod -w $sqlweight";	
     &lvslog(2,"Weight up service by: $mstring6",$curuser);
    } 
    
  
   }	
   if ($sqlenabled eq "1" ) {
    if ($sqlactive ne "1") {
    	&PrintHTMLMessage("WARNING: Host $sqlIP:$sqlIPort is in down state, continiue anyway. <BR>");
    }        #if ($sqlactive eq "1")

    if ($mstring) {
     &PrintHTMLMessage ("Apply: $mstring <BR>");
    } 
    if ($mstring6) {
     &PrintHTMLMessage ("Apply: $mstring6 <BR>");
    } 
    

    if ($simulate eq "0") {
     if ($mstring) {
      `$mstring`;
     } 
     if ($mstring6) {
      `$mstring6`;
     } 
    }
    
    &PrintHTMLMessage(" SQL: UPDATE tblNodesInventory SET weightdown = '0' where invID ='" . $sqlInvID . "' <BR>");
    if ($simulate eq "0") {
     $sqlcur = $dbh->prepare("UPDATE tblNodesInventory SET weightdown = '0' where invID ='" . $sqlInvID . "';" );
     $sqlcur->execute;
    } 
   } else {  #if ($sqlenabled eq "1" )
    &PrintHTMLMessage("SKIP (DISABLED) Apply: $mstring <BR>");     
   }         #if ($sqlenabled eq "1" ) 
  }          #while (my($sqlInvID,$sqlInvWeight,$sqlNodeID,$sqlLink,$sqlIPort)=$sqlinv->fetchrow_array) 
  print "Services IPv4: $srvCnt4<br>\n";
  print "Services IPv6: $srvCnt6<br>\n";
  print "Drain-Up Proces Complete!\n";
 }           #if ($tparam =~ m/^wu=(\d*)/){  


 #############
 #WEIGHT DOWN
 #############

 if ($tparam =~ m/^wd=(\d*)/){  
 	if ($simulate eq "1") {
 	 &PrintHTMLMessage("Simualtation Mode: !!<BR>");     
 	}	
	$showid=$1;
  $sqlalias = $dbh->prepare("SELECT alias,realip,realip6 FROM tblResources where resID = '" . $showid . "';" ); 
  $sqlalias->execute ; #Get Inventory and ID's
  (($sqlAlias,$sqlIP,$sqlIP6)=$sqlalias->fetchrow_array);
  $myIP=$sqlIP;
  $myIP6=$sqlIP6;
  $myHost=$sqlAlias;
  $sqlinv = $dbh->prepare("SELECT invID,weight,nodeID,link,port,active,method,weight,automovepoolid FROM tblNodesInventory where resID = '" . $showid . "';" ); 
  $sqlinv->execute ; #Get Inventory and ID's
  while (($sqlInvID,$sqlInvWeight,$sqlNodeID,$sqlLink,$sqlIPort,$sqlactive,$sqlmethod,$sqlweight,$sqlPoolID)=$sqlinv->fetchrow_array) {
   $sqlnoden = $dbh->prepare("SELECT service,vip,vip6 FROM tblNodes where nodeid = '" . $sqlNodeID . "';" ); 
   $sqlnoden->execute ; 
   ($NodeService,$NodeVip,$NodeVip6)=$sqlnoden->fetchrow_array ;
   
   if ($NodeService !~ /fwm/i) {
   	if (($NodeVip) and ($sqlIP)){
     $mstring = "$lvscmd -e -t $NodeVip -r $sqlIP:$sqlIPort -$sqlmethod -w 0";
     $srvCnt4++;
    } 
    
    if (($NodeVip6) and ($sqlIP6)){
     if ($NodeVip6=~/(.*):(\d+)$/) {  
      $mstring6 = "$lvscmd -e -t [$1]:$2 -r [$sqlIP6]:$sqlIPort -$sqlmethod -w 0";
      $srvCnt6++;
     } 
    }
    
    $mdstring = "$lvscmd -d -t $NodeVip -r $sqlIP:$sqlIPort";  #2.0.5 - leave untouched
   } else {
    
    if ($sqlIP) {
     $mstring = "$lvscmd -e -f $NodeVip -r $sqlIP -$sqlmethod -w 0";	
    } 

    if ($sqlIP6) {
     $mstring6 = "$lvscmd -e -f $NodeVip6 -r [$sqlIP] -$sqlmethod -w 0";	
    } 
    
    
    $mdstring = "$lvscmd -d -f $NodeVip -r $sqlIP";	#2.0.5 - leave untouched
   }	

    if ($mstring){
     &PrintHTMLMessage("Apply: $mstring <BR>");
     &lvslog(2,"Weight down service by: $mstring",$curuser);
    } 
    if ($mstring6){
     &PrintHTMLMessage("Apply: $mstring6 <BR>");
     &lvslog(2,"Weight down service by: $mstring6",$curuser);
    } 
   
   if ($simulate eq "0") {
    if ($mstring){
     `$mstring`;
    } 
    if ($mstring6){
     `$mstring6`;
    } 
   }
   
   
   &PrintHTMLMessage(" SQL: UPDATE tblNodesInventory SET weightdown = '1' where invID ='" . $sqlInvID . "' <BR>");
   if ($simulate eq "0") {
    $sqlcur = $dbh->prepare("UPDATE tblNodesInventory SET weightdown = '1' where invID ='" . $sqlInvID . "';" );
    $sqlcur->execute;
   } 
   $auto_ok=1;
   ##########################################
   ##############   AUTOMOVE   ##############
   ##########################################
   if (($sqlPoolID) and ($sqlPoolID ne "0")) {
   	$auto_ok=undef;
    &PrintHTMLMessage("ATTENTION: Automovemove Service to: $sqlPoolID <BR>");
    &PrintHTMLMessage("SQL: SELECT resID FROM tblPoolResources where poolID = '" . $sqlPoolID . "' <BR>");
    $sqlpool = $dbh->prepare("SELECT resID FROM tblPoolResources where poolID = '" . $sqlPoolID . "';" ); 
    $sqlpool->execute ; #Get Inventory and ID's
    while (($sqlPoolReource)=$sqlpool->fetchrow_array) {
     if ($sqlPoolReource ne $showid) {
      &PrintHTMLMessage("SQL: SELECT realip,enabled,alias FROM tblResources where resID = '" . $sqlPoolReource . "'  <BR>");
      $sqlalias = $dbh->prepare("SELECT realip,enabled,alias FROM tblResources where resID = '" . $sqlPoolReource . "';" ); 
      $sqlalias->execute ; #Get Inventory and ID's
      (my($sqlIP,$sqlAliasEnabled,$sqlHostName)=$sqlalias->fetchrow_array);
      &PrintHTMLMessage("SQL: SELECT resID from tblNodesInventory  where resID ='" . $sqlPoolReource . "' and active='1' and weightdown='0';<BR>");
      my $sqlset = $dbh->prepare("SELECT resID from tblNodesInventory  where resID ='" . $sqlPoolReource . "' and active='1' and weightdown='0';");
      $sqlset->execute ; 
      (($myCurID)=$sqlset->fetchrow_array);
      &PrintHTMLMessage("CurID: $myCurID <BR>");
      if ($myCurID) {
       if ($sqlAliasEnabled eq "1") {
       	#print "Next CurID: $myCurID <BR>";
        $weightdownstate=undef;
        &PrintHTMLMessage("Check State: SELECT invID FROM tblNodesInventory where weightdown = '1' and resID = '" . $sqlPoolReource . "';" . "<BR>");
        $sqlinv = $dbh->prepare("SELECT invID FROM tblNodesInventory where weightdown = '1' and resID = '" . $sqlPoolReource . "';" ); 
        $sqlinv->execute ; #Get Inventory and ID's
        while (($sqlInvTID)=$sqlinv->fetchrow_array) {
         $weightdownstate=1;  
        }
        &PrintHTMLMessage("State: $weightdownstate <BR>");
        if (!$weightdownstate) {
         &PrintHTMLMessage("Resource is good for takeover $sqlIP -- $sqlPoolReource <BR>");
         $myCHost=$sqlHostName;
         $auto_ok=1;
    	   if ($NodeService !~ /fwm/i) {
          $mxstring = "$lvscmd -a -t $NodeVip -r $sqlIP:$sqlIPort -$sqlmethod -w $sqlweight";
         } else {  
          $mxstring = "$lvscmd -a -f $NodeVip -r $sqlIP -$sqlmethod -w $sqlweight";	
         }	
         &PrintHTMLMessage("ReApply: $mdstring <BR>");
         if ($simulate eq "0") {
          `$mxstring`;
         } 
         &PrintHTMLMessage("ReApply: $mxstring <BR>");
         if ($simulate eq "0") {
          `$mdstring`;
         } 
         &lvslog(2,"Move service by service by: $mxstring",$curuser);
         &lvslog(2,"Move service by service by: $mdstring",$curuser);
         #Set New Node
         &PrintHTMLMessage("SQL: UPDATE tblNodesInventory SET resID = '" . $sqlPoolReource . "' where invID ='" . $sqlInvID . "' <BR>");
         if ($simulate eq "0") {
          $sqlcur = $dbh->prepare("UPDATE tblNodesInventory SET resID = '" . $sqlPoolReource . "' where invID ='" . $sqlInvID . "';" );
          $sqlcur->execute;
         } 
         &PrintHTMLMessage(" SQL: UPDATE tblNodesInventory SET enabled = '1' where invID ='" . $sqlInvID . "' <BR>");
         if ($simulate eq "0") {
          $sqlcur = $dbh->prepare("UPDATE tblNodesInventory SET enabled = '1' where invID ='" . $sqlInvID . "';" );
          $sqlcur->execute;
         } 
         &PrintHTMLMessage(" SQL: UPDATE tblNodesInventory SET weightdown = '0' where invID ='" . $sqlInvID . "' <BR>");
         if ($simulate eq "0") {
          $sqlcur = $dbh->prepare("UPDATE tblNodesInventory SET weightdown = '0' where invID ='" . $sqlInvID . "';" );
          $sqlcur->execute;
         } 
         ($var,$value) = split(/$myHost\s*/,$sqlLink,2);     #Get Full Command 
         &PrintHTMLMessage(" SQL: UPDATE tblNodesInventory SET link = '".$var.$myCHost.$value."' where invID ='" . $sqlInvID . "' <BR>");
         if ($simulate eq "0") {
          $sqlcur = $dbh->prepare("UPDATE tblNodesInventory SET link = '".$var.$myCHost.$value."' where invID ='" . $sqlInvID . "';" );
          $sqlcur->execute;
         } 
         last;
        } else { #if (!$weightdownstate) { 
         &PrintHTMLMessage("WARNING: Resource is bad for takeover -- $sqlPoolReource <BR>");
        } #if (!$weightdownstate) { 
       }  #if ($sqlAliasEnabled eq "1") {
      } 
     }	 #if ($sqlPoolReource ne $showid)  
    }    #while (($sqlPoolReource)=$sqlpool->fetchrow_array) 	    
    if (!$auto_ok) {
     &PrintHTMLMessage("==>> CRITICAL <<== - No Server found for automove <BR>");
   	 #SendEmergencyEMail
    }
   }     #if (($sqlPoolID) and ($sqlPoolID ne "0"))
   ##########################################
   ###########  DONE AUTOMOVE  ##############
   ##########################################
  }   #while (my($sqlInvID,$sqlInvWeight,$sqlNodeID,$sqlLink,$sqlIPort)=$sqlinv->fetchrow_array) 
  print "Services IPv4: $srvCnt4<br>\n";
  print "Services IPv6: $srvCnt6<br>\n";
  print "Drain-Down Process Complete!\n";
 }    #if ($tparam =~ m/^wd=(\d*)/)  
} else {     #if ($granted) 
  print "No Permissions To Perform This Task !\n";
}            #if ($granted)  
print &HTMLEndInsideTable;
print &HTMLEndOutsideTable;
print &HTMLEndHeader;
 
 
sub getconfig {
 #Read xml config file
 my $xml = new XML::Simple (KeyAttr=>[]); # create XML object
 my $data = $xml->XMLin("$conffile");
 #print Dumper($data);
 $servername=$data->{SQLServer}->{Address};
 $database=$data->{SQLServer}->{Database};
 $sqluser=$data->{SQLServer}->{User};
 $sqlpassword=$data->{SQLServer}->{Password};
 $lvscmd=$data->{LVS}->{Command};
 $bkupservername=$data->{SQLServerBackup}->{Address};
 $bkupdatabase=$data->{SQLServerBackup}->{Database};
 $bkupsqluser=$data->{SQLServerBackup}->{User};
 $bkupsqlpassword=$data->{SQLServerBackup}->{Password};
 $bkupreplcommand=$data->{SQLServerBackup}->{ReplCommand};
 $ActiveWarning=$data->{ShowAmount}->{ActiveWarning};
 $InActiveWarning=$data->{ShowAmount}->{InActiveWarning};
 $HTTP_ERR=$data->{ShowAmount}->{HTTP_ERR};
 $HTTP_WARN=$data->{ShowAmount}->{HTTP_WARN};
 $simulate=$data->{Operation}->{WebSimulate};
 $maxcount=$data->{ShowAmount}->{MaxConCount};
 $backuppath=$data->{SQLConfigBackup}->{Path};
 $restoreapp=$data->{SQLConfigBackup}->{RestoreApp};
 $backupapp=$data->{SQLConfigBackup}->{BackupApp};
 $SuppressedAddress=$data->{StatusView}->{SupressedAddress};
 $HTMLLVSView=$data->{View}->{HTMLLVSOutput};
 $sitelogo=$data->{Logo}->{Path};
}


sub lvslog {
 #Logs into sql
 #$_[0] = Code (used for the image)
 #$_[1] = Message
 #$_[2] = User
 #my $sqlcur = $dbh->prepare("INSERT into tblLogging (code,user,day,month,year,hour,minute,second,message) VALUES('".$_[0]."','".$_[2]."','".strftime("%d", localtime(time))."','".strftime("%m", localtime(time))."','".strftime("%Y", localtime(time))."','".strftime("%H", localtime(time))."','".strftime("%M", localtime(time))."','".strftime("%S", localtime(time))."','$_[1]');" );
 my $sqlcur = $dbh->prepare("INSERT into tblLogging (code,user,day,month,year,hour,minute,second,message,epoche) VALUES('".$_[0]."','".$_[2]."','".strftime("%d", localtime(time))."','".strftime("%m", localtime(time))."','".strftime("%Y", localtime(time))."','".strftime("%H", localtime(time))."','".strftime("%M", localtime(time))."','".strftime("%S", localtime(time))."','".$_[1]."','".time."');" );
 $sqlcur->execute;
}


sub HTMLHeader {
#<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
my $ret='Content-type: text/html; charset=iso-8859-1

<html>
<head>
<title>LVS</title>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
<style type="text/css">

<!--
.fontairname {
	font-family: Tahoma;
	font-size: 11px;
	font-style: normal;
	font-weight: bold;
	color: #979696;
}
.fontnormal {
	font-family: Tahoma;
	font-size: 11px;
	font-style: normal;
	font-weight: normal;
	color: #979696;
}
.ecke-top-hg-lft {
	background-image: url(images/ecke-top-hg-lft.gif);
	background-repeat: repeat-y;
	background-position: right;
}
.whitefield {
	background-color: #FFFFFF;
	border: 1px solid #cfcfcf;
}
-->
</style>
<style type="text/css">
<!--
.fontblacksmall {
	font-family: Verdana;
	font-size: 10px;
	font-style: normal;
	font-weight: normal;
	color: #000000;
}
-->
</style>
</head>
';
	
return $ret;	
}



sub HTMLStartOutsideTable {
my $ret='
<body><br>
<!-- Aussentabelle -->
<table width="830" border="0" align="center" cellpadding="0" cellspacing="0">
  <tr> 
    <td width="33" height="12" valign="bottom"><img src="/images/ecke-top-lft.gif" width="33" height="12"></td>
    <td height="12" background="/images/hg-top.gif"><img src="/images/1x1_clear.gif" width="1" height="1"></td>
    <td width="33" height="12" valign="bottom"><img src="/images/ecke-top-rt.gif" width="33" height="12"></td>
  </tr>
  <tr> 
    <td width="33"><img src="/images/ecke-top2-lft.gif" width="33" height="29"></td>
    <td class="fontairname">LVS Inventory Manager<span class="fontnormal"> ( </span>'."User: $curuser from IP: $curip - Version: $version ".')</span></td>
    <td width="33"><img src="/images/ecke-top2-rt.gif" width="33" height="29"></td>
  </tr>
  <tr> 
    <td width="33"><img src="/images/ecke-top3-lft.gif" width="33" height="15"></td>
    <td background="/images/hg-top3.gif"><img src="/images/1x1_clear.gif" width="1" height="1"></td>
    <td width="33"><img src="/images/ecke-top3-rt.gif" width="33" height="15"></td>
  </tr>
  <tr> 
    <td width="33" background="/images/ecke-top-hg-lft.gif">&nbsp;</td>
    <td bgcolor="#f8f8f8"><!-- Supported WebFare Tabelle --><table border="0" cellspacing="0" cellpadding="0">
        <tr>
          <td width="45"><img src="/images/pc-icon.gif" width="25" height="26"></td>
          <td class="fontairname">'.$GlobalAlias.'<span class="fontnormal"> (</span><span class="fontnormal">'.$GlobalIP.'
          )</span></td>
        </tr>
      </table><!-- End of Supported WebFare Tabelle --></td>
    <td width="33" background="/images/ecke-top-hg-rt.gif">&nbsp;</td>
  </tr>
  <tr> 
    <td width="33" height="12"><img src="/images/ecke-top4-lft.gif" width="33" height="12"></td>
    <td height="12" background="/images/hg-top4.gif"><img src="/images/1x1_clear.gif" width="1" height="1"></td>
    <td width="33" height="12"><img src="/images/ecke-top4-rt.gif" width="33" height="12"></td>
  </tr>
  <tr> 
    <td width="33" background="/images/border-hg-lft.gif">&nbsp;</td>
    <td align="left" valign="top"> 
';	
return $ret;		
}

sub HTMLStartInsideTable {
my $ret='
      <!-- Hier kommt innentabelle rein ! --><br>
      <br>
      <table width="549" border="0" align="center" cellpadding="0" cellspacing="0">
        <tr> 
          <td width="19" height="11" valign="bottom"><img src="/images/t2_ecke-top-lft.gif" width="19" height="11"></td>
          <td height="11" valign="bottom" background="/images/t2_top-hg.gif"></td>
          <td width="19" height="11" valign="bottom"><img src="/images/t2_ecke-top-rt.gif" width="19" height="11"></td>
        </tr>
        <tr> 
          <td width="19" background="/images/t2_hg-lft.gif">&nbsp;</td>
          <td bgcolor="f8f8f8" class="fontairname">'.$GlobalTxt.'</td>
          <td width="19" background="/images/t2_hg-rt.gif">&nbsp;</td>
        </tr>
        <tr> 
          <td width="19" background="/images/t2_hg-lft.gif">&nbsp;</td>
          <td class="whitefield"><br>
          <table width="90%" border="0" align="center" cellpadding="0" cellspacing="0">
           <tr>
           <td class="fontblacksmall">
 ';	                
 return $ret;		
}
                
                
                
sub HTMLEndInsideTable {
my $ret='
              </td>
              </tr>
            </table>
            <br>
          </td>
          <td width="19" background="/images/t2_hg-rt.gif">&nbsp;</td>
        </tr>
        <tr> 
          <td width="19" height="19"><img src="/images/t2_ecke-bottom-lft.gif" width="19" height="19"></td>
          <td height="19" background="/images/t2_bottom-hg.gif">&nbsp;</td>
          <td width="19" height="19"><img src="/images/t2_ecke-bottom-rt.gif" width="19" height="19"></td>
        </tr>
        <tr> 
          <td colspan="2" align="left" valign="top"><a href="'."$retscriptname".'"><img src="/images/back-tab.gif" alt="previous page" width="63" height="29" border="0"></a></td>
          <td width="19">&nbsp;</td>
        </tr>
      </table>
      <!-- Innere Tabelle -->
 ';	                
 return $ret;		
}

sub HTMLEndOutsideTable {
my $ret='
    </td>
    <td width="33" background="/images/border-hg-rt.gif"><img src="/images/1x1_clear.gif" width="1" height="1"></td>
  </tr>
  <tr> 
    <td width="33" height="76"><img src="/images/ecke-bottom-lft.gif" width="33" height="76"></td>
    <td height="76" align="right" valign="top" background="/images/border-hg-bottom.gif">
      <!-- myLVS-Logo Tabelle -->
      <table border="0" cellspacing="0" cellpadding="0">
        <tr>
          <td class="fontairname"><img src="'.$sitelogo.'"></td>
        </tr>
      </table>
      <!-- End of myLVS-Logo Tabelle -->
    </td>
    <td width="33" height="76"><img src="/images/ecke-bottom-rt.gif" width="33" height="76"></td>
  </tr>
</table>
<!-- End of Aussentabelle -->
';	                
 return $ret;		
}


sub HTMLEndHeader {
my $ret='
</body>
</html>
';	                
 
 if ($prtDeb) {
  return $hdeb.$ret;		
 } else {
 	return $ret;		
 } 
}

sub chkuser {
	
	my ($dbhf,$fnd);
	$dbhf = DBI->connect("DBI:mysql:$database:$servername:3306",$sqluser,$sqlpassword); 
  my $sqlusr = $dbhf->prepare("SELECT operations FROM tblUsers where name='".$_[0]."' ;" );
  $sqlusr->execute ; 
  (($fnd)=$sqlusr->fetchrow_array);

  if ($fnd eq 1) {
	 #print "=> $_[0]-$fnd OK\n";
	 return "1";
	} else {
	 #print "=> $_[0] FAIL\n";
	 return undef;
	}
	
}	


sub PrintHTMLMessage {
	if ($HTMLLVSView){
	 print "$_[0]\n";
	} 
}